# coding=utf-8
"""
django-roughpages
"""
__author__ = 'Alisue <lambdalisue@hashnote.net>'
from app_version import get_versions
__version__, VERSION = get_versions('django-roughpages', allow_ambiguous=True)
